@@ -1,6 +1,6 @@
 /* Skipping uninteresting files and functions while stepping.
 
-   Copyright (C) 2011-2019 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "skip.h"
+#include "event-top.h"
 #include "value.h"
 #include "valprint.h"
 #include "ui-out.h"
 #include "symtab.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
 #include "command.h"
 #include "completer.h"
 #include "stack.h"
@@ -29,14 +29,15 @@
 #include "arch-utils.h"
 #include "linespec.h"
 #include "objfiles.h"
-#include "breakpoint.h" /* for get_sal_arch () */
+#include "breakpoint.h"
 #include "source.h"
 #include "filenames.h"
 #include "fnmatch.h"
-#include "gdb_regex.h"
-#include "gdbsupport/gdb_optional.h"
+#include "gdbsupport/gdb_regex.h"
+#include <optional>
 #include <list>
 #include "cli/cli-style.h"
+#include "gdbsupport/buildargv.h"
 
 /* True if we want to print debug printouts related to file/function
    skipping. */
@@ -112,7 +113,7 @@ private: /* data */
   std::string m_function;
 
   /* If this is a function regexp, the compiled form.  */
-  gdb::optional<compiled_regex> m_compiled_function_regexp;
+  std::optional<compiled_regex> m_compiled_function_regexp;
 
   /* Enabled/disabled state.  */
   bool m_enabled = true;
@@ -139,14 +140,8 @@ skiplist_entry::skiplist_entry (bool file_is_glob,
   if (m_function_is_regexp)
     {
       gdb_assert (!m_function.empty ());
-
-      int flags = REG_NOSUB;
-#ifdef REG_EXTENDED
-      flags |= REG_EXTENDED;
-#endif
-
-      gdb_assert (!m_function.empty ());
-      m_compiled_function_regexp.emplace (m_function.c_str (), flags,
+      m_compiled_function_regexp.emplace (m_function.c_str (),
+					  REG_NOSUB | REG_EXTENDED,
 					  _("regexp"));
     }
 }
@@ -189,7 +184,7 @@ skip_file_command (const char *arg, int from_tty)
   skiplist_entry::add_entry (false, std::string (filename),
 			     false, std::string ());
 
-  printf_filtered (_("File %s will be skipped when stepping.\n"), filename);
+  gdb_printf (_("File %s will be skipped when stepping.\n"), filename);
 }
 
 /* Create a skiplist entry for the given function NAME and add it to the
@@ -200,7 +195,7 @@ skip_function (const char *name)
 {
   skiplist_entry::add_entry (false, std::string (), false, std::string (name));
 
-  printf_filtered (_("Function %s will be skipped when stepping.\n"), name);
+  gdb_printf (_("Function %s will be skipped when stepping.\n"), name);
 }
 
 static void
@@ -209,18 +204,15 @@ skip_function_command (const char *arg, int from_tty)
   /* Default to the current function if no argument is given.  */
   if (arg == NULL)
     {
+      frame_info_ptr fi = get_selected_frame (_("No default function now."));
+      struct symbol *sym = get_frame_function (fi);
       const char *name = NULL;
-      CORE_ADDR pc;
 
-      if (!last_displayed_sal_is_valid ())
-	error (_("No default function now."));
-
-      pc = get_last_displayed_addr ();
-      if (!find_pc_partial_function (pc, &name, NULL, NULL))
-	{
-	  error (_("No function found containing current program point %s."),
-		  paddress (get_current_arch (), pc));
-	}
+      if (sym != NULL)
+	name = sym->print_name ();
+      else
+	error (_("No function found containing current program point %s."),
+	       paddress (get_current_arch (), get_frame_pc (fi)));
       skip_function (name);
       return;
     }
@@ -339,20 +331,20 @@ skip_command (const char *arg, int from_tty)
 
     if (function_to_print == NULL)
       {
-	printf_filtered (_("%s %s will be skipped when stepping.\n"),
-			 file_text, file_to_print);
+	gdb_printf (_("%s %s will be skipped when stepping.\n"),
+		    file_text, file_to_print);
       }
     else if (file_to_print == NULL)
       {
-	printf_filtered (_("%s %s will be skipped when stepping.\n"),
-			 function_text, function_to_print);
+	gdb_printf (_("%s %s will be skipped when stepping.\n"),
+		    function_text, function_to_print);
       }
     else
       {
-	printf_filtered (_("%s %s in %s %s will be skipped"
-			   " when stepping.\n"),
-			 function_text, function_to_print,
-			 lower_file_text, file_to_print);
+	gdb_printf (_("%s %s in %s %s will be skipped"
+		      " when stepping.\n"),
+		    function_text, function_to_print,
+		    lower_file_text, file_to_print);
       }
   }
 }
@@ -494,9 +486,9 @@ bool
 skiplist_entry::do_skip_file_p (const symtab_and_line &function_sal) const
 {
   if (debug_skip)
-    fprintf_unfiltered (gdb_stdlog,
-			"skip: checking if file %s matches non-glob %s...",
-			function_sal.symtab->filename, m_file.c_str ());
+    gdb_printf (gdb_stdlog,
+		"skip: checking if file %s matches non-glob %s...",
+		function_sal.symtab->filename, m_file.c_str ());
 
   bool result;
 
@@ -521,7 +513,7 @@ skiplist_entry::do_skip_file_p (const symtab_and_line &function_sal) const
     }
 
   if (debug_skip)
-    fprintf_unfiltered (gdb_stdlog, result ? "yes.\n" : "no.\n");
+    gdb_printf (gdb_stdlog, result ? "yes.\n" : "no.\n");
 
   return result;
 }
@@ -530,16 +522,16 @@ bool
 skiplist_entry::do_skip_gfile_p (const symtab_and_line &function_sal) const
 {
   if (debug_skip)
-    fprintf_unfiltered (gdb_stdlog,
-			"skip: checking if file %s matches glob %s...",
-			function_sal.symtab->filename, m_file.c_str ());
+    gdb_printf (gdb_stdlog,
+		"skip: checking if file %s matches glob %s...",
+		function_sal.symtab->filename, m_file.c_str ());
 
   bool result;
 
   /* Check first sole SYMTAB->FILENAME.  It may not be a substring of
      symtab_to_fullname as it may contain "./" etc.  */
   if (gdb_filename_fnmatch (m_file.c_str (), function_sal.symtab->filename,
-			    FNM_FILE_NAME | FNM_NOESCAPE) == 0)
+			    FNM_NOESCAPE) == 0)
     result = true;
 
   /* Before we invoke symtab_to_fullname, which is expensive, do a quick
@@ -550,18 +542,18 @@ skiplist_entry::do_skip_gfile_p (const symtab_and_line &function_sal) const
   else if (!basenames_may_differ
       && gdb_filename_fnmatch (lbasename (m_file.c_str ()),
 			       lbasename (function_sal.symtab->filename),
-			       FNM_FILE_NAME | FNM_NOESCAPE) != 0)
+			       FNM_NOESCAPE) != 0)
     result = false;
   else
     {
       /* Note: symtab_to_fullname caches its result, thus we don't have to.  */
       const char *fullname = symtab_to_fullname (function_sal.symtab);
 
-      result = compare_glob_filenames_for_search (fullname, m_file.c_str ());
+      result = gdb_filename_fnmatch (m_file.c_str (), fullname, FNM_NOESCAPE);
     }
 
   if (debug_skip)
-    fprintf_unfiltered (gdb_stdlog, result ? "yes.\n" : "no.\n");
+    gdb_printf (gdb_stdlog, result ? "yes.\n" : "no.\n");
 
   return result;
 }
@@ -592,9 +584,9 @@ skiplist_entry::skip_function_p (const char *function_name) const
   if (m_function_is_regexp)
     {
       if (debug_skip)
-        fprintf_unfiltered (gdb_stdlog,
-			    "skip: checking if function %s matches regex %s...",
-			    function_name, m_function.c_str ());
+	gdb_printf (gdb_stdlog,
+		    "skip: checking if function %s matches regex %s...",
+		    function_name, m_function.c_str ());
 
       gdb_assert (m_compiled_function_regexp);
       result
@@ -603,15 +595,15 @@ skiplist_entry::skip_function_p (const char *function_name) const
   else
     {
       if (debug_skip)
-        fprintf_unfiltered (gdb_stdlog,
-			    ("skip: checking if function %s matches non-regex "
-			     "%s..."),
-			    function_name, m_function.c_str ());
+	gdb_printf (gdb_stdlog,
+		    ("skip: checking if function %s matches non-regex "
+		     "%s..."),
+		    function_name, m_function.c_str ());
       result = (strcmp_iw (function_name, m_function.c_str ()) == 0);
     }
 
   if (debug_skip)
-    fprintf_unfiltered (gdb_stdlog, result ? "yes.\n" : "no.\n");
+    gdb_printf (gdb_stdlog, result ? "yes.\n" : "no.\n");
 
   return result;
 }
@@ -659,14 +651,13 @@ complete_skip_number (cmd_list_element *cmd,
 
   for (const skiplist_entry &entry : skiplist_entries)
     {
-      gdb::unique_xmalloc_ptr<char> name (xstrprintf ("%d", entry.number ()));
+      gdb::unique_xmalloc_ptr<char> name = xstrprintf ("%d", entry.number ());
       if (strncmp (word, name.get (), word_len) == 0)
 	completer.add_completion (std::move (name));
     }
 }
 
-void
-_initialize_step_skip (void)
+INIT_GDB_FILE (step_skip)
 {
   static struct cmd_list_element *skiplist = NULL;
   struct cmd_list_element *c;
@@ -684,14 +675,14 @@ FILE-SPEC is one of:\n\
 FUNCTION-SPEC is one of:\n\
        -fu|-function FUNCTION-NAME\n\
        -rfu|-rfunction FUNCTION-NAME-REGULAR-EXPRESSION"),
-                  &skiplist, "skip ", 1, &cmdlist);
+		  &skiplist, 1, &cmdlist);
 
   c = add_cmd ("file", class_breakpoint, skip_file_command, _("\
 Ignore a file while stepping.\n\
 Usage: skip file [FILE-NAME]\n\
 If no filename is given, ignore the current file."),
 	       &skiplist);
-  set_cmd_completer (c, filename_completer);
+  set_cmd_completer (c, deprecated_filename_completer);
 
   c = add_cmd ("function", class_breakpoint, skip_function_command, _("\
 Ignore a function while stepping.\n\
@@ -730,7 +721,7 @@ If you don't specify any numbers or ranges, we'll delete all skip entries."),
   add_info ("skip", info_skip_command, _("\
 Display the status of skips.\n\
 Usage: info skip [NUMBER | RANGES]...\n\
-You can specify numbers (e.g. \"info skip 1 3\"), \n\
+You can specify numbers (e.g. \"info skip 1 3\"),\n\
 ranges (e.g. \"info skip 4-8\"), or both (e.g. \"info skip 1 3 4-8\").\n\n\
 If you don't specify any numbers or ranges, we'll show all skips."));
   set_cmd_completer (c, complete_skip_number);
